"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelApi = void 0;
const nodes_1 = require("./nodes");
const PatchBuilder_1 = require("../../../json-crdt-patch/PatchBuilder");
class ModelApi {
    constructor(model) {
        this.model = model;
        this.next = 0;
        this.builder = new PatchBuilder_1.PatchBuilder(this.model.clock);
    }
    get node() {
        return new nodes_1.NodeApi(this.model.root.node(), this);
    }
    get r() {
        return new nodes_1.NodeApi(this.model.root, this);
    }
    find(path) {
        return this.node.find(path);
    }
    val(path) {
        return this.node.val(path);
    }
    str(path) {
        return this.node.str(path);
    }
    bin(path) {
        return this.node.bin(path);
    }
    arr(path) {
        return this.node.arr(path);
    }
    obj(path) {
        return this.node.obj(path);
    }
    const(path) {
        return this.node.const(path);
    }
    root(json) {
        const builder = this.builder;
        builder.root(builder.json(json));
        this.apply();
        return this;
    }
    apply() {
        const ops = this.builder.patch.ops;
        const length = ops.length;
        for (let i = this.next; i < length; i++)
            this.model.applyOperation(ops[i]);
        this.next = length;
    }
    advance() {
        this.next = this.builder.patch.ops.length;
    }
    view() {
        return this.model.view();
    }
    flush() {
        const patch = this.builder.flush();
        this.next = 0;
        return patch;
    }
}
exports.ModelApi = ModelApi;
