"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelApi = void 0;
var nodes_1 = require("./nodes");
var PatchBuilder_1 = require("../../../json-crdt-patch/PatchBuilder");
var ModelApi = (function () {
    function ModelApi(model) {
        this.model = model;
        this.next = 0;
        this.builder = new PatchBuilder_1.PatchBuilder(this.model.clock);
    }
    Object.defineProperty(ModelApi.prototype, "node", {
        get: function () {
            return new nodes_1.NodeApi(this.model.root.node(), this);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ModelApi.prototype, "r", {
        get: function () {
            return new nodes_1.NodeApi(this.model.root, this);
        },
        enumerable: false,
        configurable: true
    });
    ModelApi.prototype.find = function (path) {
        return this.node.find(path);
    };
    ModelApi.prototype.val = function (path) {
        return this.node.val(path);
    };
    ModelApi.prototype.str = function (path) {
        return this.node.str(path);
    };
    ModelApi.prototype.bin = function (path) {
        return this.node.bin(path);
    };
    ModelApi.prototype.arr = function (path) {
        return this.node.arr(path);
    };
    ModelApi.prototype.obj = function (path) {
        return this.node.obj(path);
    };
    ModelApi.prototype.const = function (path) {
        return this.node.const(path);
    };
    ModelApi.prototype.root = function (json) {
        var builder = this.builder;
        builder.root(builder.json(json));
        this.apply();
        return this;
    };
    ModelApi.prototype.apply = function () {
        var ops = this.builder.patch.ops;
        var length = ops.length;
        for (var i = this.next; i < length; i++)
            this.model.applyOperation(ops[i]);
        this.next = length;
    };
    ModelApi.prototype.advance = function () {
        this.next = this.builder.patch.ops.length;
    };
    ModelApi.prototype.view = function () {
        return this.model.view();
    };
    ModelApi.prototype.flush = function () {
        var patch = this.builder.flush();
        this.next = 0;
        return patch;
    };
    return ModelApi;
}());
exports.ModelApi = ModelApi;
