"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValueApi = exports.StringApi = exports.ObjectApi = exports.ConstApi = exports.BinaryApi = exports.ArrayApi = exports.NodeApi = void 0;
var tslib_1 = require("tslib");
var ArrayRga_1 = require("../../types/rga-array/ArrayRga");
var BinaryRga_1 = require("../../types/rga-binary/BinaryRga");
var Const_1 = require("../../types/const/Const");
var find_1 = require("./find");
var clock_1 = require("../../../json-crdt-patch/clock");
var ObjectLww_1 = require("../../types/lww-object/ObjectLww");
var StringRga_1 = require("../../types/rga-string/StringRga");
var ValueLww_1 = require("../../types/lww-value/ValueLww");
var NodeApi = (function () {
    function NodeApi(node, api) {
        this.node = node;
        this.api = api;
    }
    NodeApi.prototype.find = function (path) {
        var node = this.node;
        if (path === undefined) {
            if (typeof node.child === 'function') {
                var child = node.child();
                if (!child)
                    throw new Error('NO_CHILD');
                return child;
            }
            throw new Error('CANNOT_IN');
        }
        if (typeof path === 'string' && !!path && path[0] !== '/')
            path = '/' + path;
        if (typeof path === 'number')
            path = [path];
        return (0, find_1.find)(this.node, path);
    };
    NodeApi.prototype.in = function (path) {
        var node = this.find(path);
        if (node instanceof ValueLww_1.ValueLww)
            return new ValueApi(node, this.api);
        else if (node instanceof StringRga_1.StringRga)
            return new StringApi(node, this.api);
        else if (node instanceof BinaryRga_1.BinaryRga)
            return new BinaryApi(node, this.api);
        else if (node instanceof ArrayRga_1.ArrayRga)
            return new ArrayApi(node, this.api);
        else if (node instanceof ObjectLww_1.ObjectLww)
            return new ObjectApi(node, this.api);
        else if (node instanceof Const_1.Const)
            return new ConstApi(node, this.api);
        else
            throw new Error('UNKNOWN_NODE');
    };
    NodeApi.prototype.asVal = function () {
        if (this.node instanceof ValueLww_1.ValueLww)
            return new ValueApi(this.node, this.api);
        throw new Error('NOT_VAL');
    };
    NodeApi.prototype.asStr = function () {
        if (this.node instanceof StringRga_1.StringRga)
            return new StringApi(this.node, this.api);
        throw new Error('NOT_STR');
    };
    NodeApi.prototype.asBin = function () {
        if (this.node instanceof BinaryRga_1.BinaryRga)
            return new BinaryApi(this.node, this.api);
        throw new Error('NOT_BIN');
    };
    NodeApi.prototype.asArr = function () {
        if (this.node instanceof ArrayRga_1.ArrayRga)
            return new ArrayApi(this.node, this.api);
        throw new Error('NOT_ARR');
    };
    NodeApi.prototype.asObj = function () {
        if (this.node instanceof ObjectLww_1.ObjectLww)
            return new ObjectApi(this.node, this.api);
        throw new Error('NOT_OBJ');
    };
    NodeApi.prototype.asConst = function () {
        if (this.node instanceof Const_1.Const)
            return new ConstApi(this.node, this.api);
        throw new Error('NOT_CONST');
    };
    NodeApi.prototype.val = function (path) {
        return this.in(path).asVal();
    };
    NodeApi.prototype.str = function (path) {
        return this.in(path).asStr();
    };
    NodeApi.prototype.bin = function (path) {
        return this.in(path).asBin();
    };
    NodeApi.prototype.arr = function (path) {
        return this.in(path).asArr();
    };
    NodeApi.prototype.obj = function (path) {
        return this.in(path).asObj();
    };
    NodeApi.prototype.const = function (path) {
        return this.in(path).asConst();
    };
    NodeApi.prototype.view = function () {
        return this.node.view();
    };
    return NodeApi;
}());
exports.NodeApi = NodeApi;
var ArrayApi = (function (_super) {
    tslib_1.__extends(ArrayApi, _super);
    function ArrayApi() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ArrayApi.prototype.ins = function (index, values) {
        var _a = this, api = _a.api, node = _a.node;
        var builder = api.builder;
        var after = !index ? node.id : node.find(index - 1);
        if (!after)
            throw new Error('OUT_OF_BOUNDS');
        var valueIds = [];
        for (var i = 0; i < values.length; i++)
            valueIds.push(builder.json(values[i]));
        builder.insArr(node.id, after, valueIds);
        api.apply();
        return this;
    };
    ArrayApi.prototype.del = function (index, length) {
        var _a = this, api = _a.api, node = _a.node;
        var spans = node.findInterval(index, length);
        if (!spans)
            throw new Error('OUT_OF_BOUNDS');
        api.builder.del(node.id, spans);
        api.apply();
        return this;
    };
    return ArrayApi;
}(NodeApi));
exports.ArrayApi = ArrayApi;
var BinaryApi = (function (_super) {
    tslib_1.__extends(BinaryApi, _super);
    function BinaryApi() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BinaryApi.prototype.ins = function (index, data) {
        var _a = this, api = _a.api, node = _a.node;
        var after = !index ? node.id : node.find(index - 1);
        if (!after)
            throw new Error('OUT_OF_BOUNDS');
        api.builder.insBin(node.id, after, data);
        api.apply();
        return this;
    };
    BinaryApi.prototype.del = function (index, length) {
        var _a = this, api = _a.api, node = _a.node;
        var spans = node.findInterval(index, length);
        if (!spans)
            throw new Error('OUT_OF_BOUNDS');
        api.builder.del(node.id, spans);
        api.apply();
        return this;
    };
    return BinaryApi;
}(NodeApi));
exports.BinaryApi = BinaryApi;
var ConstApi = (function (_super) {
    tslib_1.__extends(ConstApi, _super);
    function ConstApi() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ConstApi.prototype.view = function () {
        return this.node.view();
    };
    return ConstApi;
}(NodeApi));
exports.ConstApi = ConstApi;
var ObjectApi = (function (_super) {
    tslib_1.__extends(ObjectApi, _super);
    function ObjectApi() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ObjectApi.prototype.set = function (entries) {
        var _a = this, api = _a.api, node = _a.node;
        var builder = api.builder;
        builder.setKeys(node.id, Object.entries(entries).map(function (_a) {
            var _b = tslib_1.__read(_a, 2), key = _b[0], json = _b[1];
            return [key, builder.constOrJson(json)];
        }));
        api.apply();
        return this;
    };
    ObjectApi.prototype.del = function (keys) {
        var _a = this, api = _a.api, node = _a.node;
        var builder = api.builder;
        api.builder.setKeys(node.id, keys.map(function (key) { return [key, builder.const(undefined)]; }));
        api.apply();
        return this;
    };
    return ObjectApi;
}(NodeApi));
exports.ObjectApi = ObjectApi;
var StringApi = (function (_super) {
    tslib_1.__extends(StringApi, _super);
    function StringApi() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StringApi.prototype.ins = function (index, text) {
        var _a = this, api = _a.api, node = _a.node;
        var nextTime = api.builder.nextTime();
        var after = node.insAt(index, (0, clock_1.ts)(api.builder.clock.sid, nextTime), text);
        if (!after)
            throw new Error('OUT_OF_BOUNDS');
        api.builder.insStr(node.id, after, text);
        api.advance();
        return this;
    };
    StringApi.prototype.del = function (index, length) {
        var _a = this, api = _a.api, node = _a.node;
        var spans = node.findInterval(index, length);
        if (!spans)
            throw new Error('OUT_OF_BOUNDS');
        node.delete(spans);
        api.builder.del(node.id, spans);
        api.advance();
        return this;
    };
    return StringApi;
}(NodeApi));
exports.StringApi = StringApi;
var ValueApi = (function (_super) {
    tslib_1.__extends(ValueApi, _super);
    function ValueApi() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ValueApi.prototype.set = function (json) {
        var _a = this, api = _a.api, node = _a.node;
        var builder = api.builder;
        var val = builder.constOrJson(json);
        api.builder.setVal(node.id, val);
        api.apply();
        return this;
    };
    return ValueApi;
}(NodeApi));
exports.ValueApi = ValueApi;
