"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpTree = exports.Register = exports.DropNode = exports.PickNode = void 0;
var tslib_1 = require("tslib");
var toStringNode = function (self, tab) {
    var e_1, _a;
    if (tab === void 0) { tab = ''; }
    var children = '';
    var last = self.children.size - 1;
    var i = 0;
    try {
        for (var _b = tslib_1.__values(self.children), _c = _b.next(); !_c.done; _c = _b.next()) {
            var _d = tslib_1.__read(_c.value, 2), index = _d[0], node = _d[1];
            var isLast = i === last;
            children += "\n".concat(tab).concat(isLast ? ' └─ ' : ' ├─ ').concat(node.toString(tab + (isLast ? '    ' : ' │  ')));
            i++;
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    var indexFormatted = typeof self.index === 'number' ? "[".concat(self.index, "]") : "\"".concat(self.index, "\"");
    var registerFormatted = self.regId === undefined
        ? ''
        : self.regId === -1
            ? ''
            : " [".concat(self.regId, "]");
    return "".concat(indexFormatted, " ").concat(self.constructor.name).concat(registerFormatted).concat(children);
};
var PickNode = (function () {
    function PickNode(index, path, pathLength) {
        this.regId = -1;
        this.index = index;
        this.path = path;
        this.pathLength = pathLength;
        this.parent = null;
        this.children = new Map();
    }
    PickNode.prototype.toString = function (tab) {
        if (tab === void 0) { tab = ''; }
        return toStringNode(this, tab);
    };
    return PickNode;
}());
exports.PickNode = PickNode;
var DropNode = (function () {
    function DropNode(index, path, pathLength) {
        this.regId = -1;
        this.index = index;
        this.path = path;
        this.pathLength = pathLength;
        this.parent = null;
        this.children = new Map();
    }
    DropNode.prototype.toString = function (tab) {
        if (tab === void 0) { tab = ''; }
        return toStringNode(this, tab);
    };
    DropNode.prototype.clone = function () {
        var e_2, _a;
        var clone = new DropNode(this.index, this.path, this.pathLength);
        clone.regId = this.regId;
        try {
            for (var _b = tslib_1.__values(this.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), index = _d[0], node = _d[1];
                var child = node.clone();
                child.parent = clone;
                clone.children.set(index, child);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return clone;
    };
    return DropNode;
}());
exports.DropNode = DropNode;
var PickRoot = (function (_super) {
    tslib_1.__extends(PickRoot, _super);
    function PickRoot() {
        return _super.call(this, 0, [], 0) || this;
    }
    return PickRoot;
}(PickNode));
var DropRoot = (function (_super) {
    tslib_1.__extends(DropRoot, _super);
    function DropRoot() {
        return _super.call(this, 0, [], 0) || this;
    }
    return DropRoot;
}(DropNode));
var Register = (function () {
    function Register(id) {
        this.id = id;
        this.pick = null;
        this.data = undefined;
        this.drops = [];
    }
    Register.prototype.addDrop = function (drop) {
        this.drops.push(drop);
    };
    Register.prototype.removeDrop = function (drop) {
        var index = this.drops.findIndex(function (v) { return v === drop; });
        if (index > -1)
            this.drops.splice(index, 1);
    };
    Register.prototype.toString = function () {
        var drops = '';
        if (this.drops.length)
            drops = this.drops.map(function (drop) { return '/' + drop.path.slice(0, drop.pathLength).join('/'); }).join(', ');
        var src = this.pick
            ? '/' + this.pick.path.slice(0, this.pick.pathLength).join('/')
            : "{ ".concat(JSON.stringify(this.data), " }");
        var dst = drops ? "{ ".concat(drops, " }") : '∅';
        return "".concat(this.id, ": ").concat(this.constructor.name, " ").concat(src, " \u2508\u2508\u2508\u2192 ").concat(dst);
    };
    return Register;
}());
exports.Register = Register;
var OpTree = (function () {
    function OpTree() {
        this.maxRegId = -1;
        this.test = [];
        this.pick = new PickRoot();
        this.drop = new DropRoot();
        this.register = new Map();
    }
    OpTree.from = function (op) {
        var _a;
        var _b = tslib_1.__read(op, 5), test = _b[0], _c = _b[1], pick = _c === void 0 ? [] : _c, _d = _b[2], data = _d === void 0 ? [] : _d, _e = _b[3], drop = _e === void 0 ? [] : _e, _f = _b[4], edit = _f === void 0 ? [] : _f;
        var tree = new OpTree();
        if (test.length)
            (_a = tree.test).push.apply(_a, tslib_1.__spreadArray([], tslib_1.__read(test), false));
        for (var i = 0; i < pick.length; i++) {
            var _g = tslib_1.__read(pick[i], 2), registerId = _g[0], what = _g[1];
            tree.addPickNode(registerId, what);
        }
        for (var i = 0; i < data.length; i++) {
            var _h = tslib_1.__read(data[i], 2), registerId = _h[0], value = _h[1];
            tree.addData(registerId, value);
        }
        for (var i = 0; i < drop.length; i++) {
            var _j = tslib_1.__read(drop[i], 2), registerId = _j[0], where = _j[1];
            tree.addDropNode(registerId, where);
        }
        return tree;
    };
    OpTree.prototype.findPick = function (path, pathLength) {
        var parent = this.pick;
        for (var i = 0; i < pathLength; i++) {
            var index = path[i];
            if (!parent.children.has(index))
                return undefined;
            parent = parent.children.get(index);
        }
        return parent;
    };
    OpTree.prototype.findDrop = function (path, pathLength) {
        var parent = this.drop;
        for (var i = 0; i < pathLength; i++) {
            var index = path[i];
            if (!parent.children.has(index))
                return undefined;
            parent = parent.children.get(index);
        }
        return parent;
    };
    OpTree.prototype.setRegister = function (register) {
        var regId = register.id;
        if (regId > this.maxRegId)
            this.maxRegId = regId;
        this.register.set(regId, register);
    };
    OpTree.prototype.addPickNode = function (registerId, what, length) {
        if (length === void 0) { length = what.length; }
        var parent = this.pick;
        if (!length) {
            parent.regId = registerId;
            var register = new Register(registerId);
            register.pick = parent;
            this.setRegister(register);
        }
        else {
            for (var i = 0; i < length; i++) {
                var index = what[i];
                var childExists = parent.children.has(index);
                if (!childExists) {
                    var child_1 = new PickNode(index, what, i + 1);
                    parent.children.set(index, child_1);
                    child_1.parent = parent;
                }
                var child = parent.children.get(index);
                var isLast = i === length - 1;
                if (isLast) {
                    if (child.regId < 0) {
                        child.regId = registerId;
                        var register = new Register(registerId);
                        register.pick = child;
                        this.setRegister(register);
                    }
                }
                parent = child;
            }
        }
        return this.register.get(parent.regId);
    };
    OpTree.prototype.addData = function (registerId, data) {
        var register = new Register(registerId);
        register.data = data;
        this.setRegister(register);
    };
    OpTree.prototype.addDropNode = function (registerId, where) {
        var parent = this.drop;
        var length = where.length;
        if (!length) {
            var register = this.register.get(registerId);
            if (register instanceof Register) {
                parent.regId = register.id;
                register.addDrop(parent);
            }
        }
        else {
            for (var i = 0; i < length; i++) {
                var index = where[i];
                var childExists = parent.children.has(index);
                if (!childExists) {
                    var child_2 = new DropNode(index, where, i + 1);
                    parent.children.set(index, child_2);
                    child_2.parent = parent;
                }
                var child = parent.children.get(index);
                var isLast = i === length - 1;
                if (isLast) {
                    child.regId = registerId;
                    var register = this.register.get(registerId);
                    register.addDrop(child);
                }
                parent = child;
            }
        }
    };
    OpTree.prototype.compose = function (other) {
        var _a;
        var _this = this;
        (_a = this.test).push.apply(_a, tslib_1.__spreadArray([], tslib_1.__read(other.test), false));
        var d1 = this.drop;
        var d2 = other.drop;
        other.register.forEach(function (register2) {
            if (register2.pick) {
                var path = register2.pick.path;
                var pathLength = register2.pick.pathLength;
                var deepestDropNodeInPath = _this.findDeepestDropInPath(register2.pick.path, register2.pick.pathLength);
                if (deepestDropNodeInPath) {
                    if (deepestDropNodeInPath) {
                        var dropRegister = _this.register.get(deepestDropNodeInPath.regId);
                        if (dropRegister.pick) {
                            path = tslib_1.__spreadArray(tslib_1.__spreadArray([], tslib_1.__read(dropRegister.pick.path.slice(0, dropRegister.pick.pathLength)), false), tslib_1.__read(register2.pick.path.slice(0, register2.pick.pathLength).slice(deepestDropNodeInPath.pathLength)), false);
                            pathLength = path.length;
                            register2.pick.path = path;
                            register2.pick.pathLength = pathLength;
                        }
                    }
                }
                var _loop_1 = function (i) {
                    var comp = path[i];
                    if (typeof comp === 'number') {
                        var pick = _this.findPick(path, i);
                        if (pick) {
                            var numberOfPickWithLowerIndex_1 = 0;
                            pick.children.forEach(function (child, index) {
                                if (index <= comp)
                                    numberOfPickWithLowerIndex_1++;
                            });
                            path[i] += numberOfPickWithLowerIndex_1;
                        }
                    }
                };
                for (var i = 0; i < pathLength; i++) {
                    _loop_1(i);
                }
                var isDelete = !register2.drops.length;
                if (isDelete) {
                    var op1Pick = _this.findPick(register2.pick.path, register2.pick.pathLength);
                    if (op1Pick && op1Pick.regId) {
                        var register = _this.register.get(op1Pick.regId);
                        var alreadyDeletedInOp1 = register && !register.drops.length;
                        if (alreadyDeletedInOp1)
                            return;
                    }
                    _this.addPickNode(_this.maxRegId + 1, path, pathLength);
                    var drop = _this.findDrop(register2.pick.path, register2.pick.pathLength);
                    if (drop) {
                        if (drop.parent) {
                            drop.parent.children.delete(drop.index);
                            drop.parent = null;
                            var register1 = _this.register.get(drop.regId);
                            if (register1 instanceof Register) {
                                register1.removeDrop(drop);
                                if (!register1.drops.length && !register1.pick)
                                    _this.register.delete(drop.regId);
                            }
                        }
                        else {
                            _this.drop.regId = -1;
                        }
                    }
                }
            }
        });
        this.composeDrops(d1, d2, other);
    };
    OpTree.prototype.findDeepestDropInPath = function (path, pathLength) {
        if (pathLength === void 0) { pathLength = path.length; }
        var longest = null;
        var curr = this.drop;
        for (var i = 0; i < pathLength; i++) {
            var comp = path[i];
            var child = curr.children.get(comp);
            if (!child)
                break;
            curr = child;
            if (curr.regId >= 0)
                longest = curr;
        }
        return longest;
    };
    OpTree.prototype.removeDrop = function (drop) {
        if (drop.regId >= 0) {
            var register = this.register.get(drop.regId);
            register.removeDrop(drop);
            if (!register.drops.length && !register.pick)
                this.register.delete(drop.regId);
        }
    };
    OpTree.prototype.composeDrops = function (d1, d2, tree2) {
        var e_3, _a;
        var isDrop = d2.regId >= 0;
        if (isDrop) {
            var isRoot = !d2.parent;
            var clone = !isRoot ? d2.clone() : this.drop;
            var register2 = tree2.register.get(d2.regId);
            var isDataDrop = register2.data !== undefined;
            if (isDataDrop) {
                var newRegister = new Register(this.maxRegId + 1);
                newRegister.data = register2.data;
                newRegister.addDrop(clone);
                this.setRegister(newRegister);
                clone.regId = newRegister.id;
            }
            else {
                var samePickInOp1Exists = this.findPick(register2.pick.path, register2.pick.pathLength);
                if (samePickInOp1Exists) {
                    clone.regId = samePickInOp1Exists.regId;
                }
                else {
                    var reg = this.addPickNode(this.maxRegId + 1, register2.pick.path, register2.pick.pathLength);
                    reg.addDrop(clone);
                    clone.regId = reg.id;
                }
            }
            if (!!d1.parent && !!d2.parent) {
                var child = d1.parent.children.get(d1.index);
                if (child)
                    this.removeDrop(child);
                d1.parent.children.set(d2.index, clone);
            }
        }
        try {
            for (var _b = tslib_1.__values(d2.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), index = _d[0], child2 = _d[1];
                if (!d1.children.has(index)) {
                    var child1_1 = new DropNode(child2.index, child2.path, child2.pathLength);
                    child1_1.parent = d1;
                    d1.children.set(child1_1.index, child1_1);
                }
                var child1 = d1.children.get(index);
                this.composeDrops(child1, child2, tree2);
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
    };
    OpTree.prototype.toJson = function () {
        var e_4, _a;
        var pick = [];
        var data = [];
        var drop = [];
        try {
            for (var _b = tslib_1.__values(this.register), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), index = _d[0], register = _d[1];
                if (register.data !== undefined) {
                    data.push([index, register.data]);
                }
                else {
                    var pickPath = register.pick.path;
                    var pickPathLength = register.pick.pathLength;
                    pick.push([index, pickPath.slice(0, pickPathLength)]);
                }
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_4) throw e_4.error; }
        }
        this.pushDropNode(drop, this.drop);
        return [this.test, pick, data, drop, []];
    };
    OpTree.prototype.pushDropNode = function (drop, node) {
        var _this = this;
        if (node.regId >= 0)
            drop.push([node.regId, node.path.slice(0, node.pathLength)]);
        node.children.forEach(function (child) {
            _this.pushDropNode(drop, child);
        });
    };
    OpTree.prototype.toString = function (tab) {
        var e_5, _a;
        if (tab === void 0) { tab = ''; }
        var picks = this.pick ? this.pick.toString(tab + '│  ') : ' [empty]';
        var registers = 'Registers';
        var lastRegister = this.register.size - 1;
        var i = 0;
        try {
            for (var _b = tslib_1.__values(this.register), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), id = _d[0], register = _d[1];
                var isLast = i === lastRegister;
                registers += "\n".concat(tab).concat(isLast ? '│  └─' : '│  ├─', " ").concat(register);
                i++;
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_5) throw e_5.error; }
        }
        var drops = this.drop ? this.drop.toString(tab + '   ') : ' [empty]';
        return "".concat(this.constructor.name, "\n").concat(tab, "\u251C\u2500 ").concat(picks, "\n").concat(tab, "\u2502\n").concat(tab, "\u251C\u2500 ").concat(registers, "\n").concat(tab, "\u2502\n").concat(tab, "\u2514\u2500 ").concat(drops);
    };
    return OpTree;
}());
exports.OpTree = OpTree;
